/***************************************************************************
 *   Copyright (C) 2007 by Anatoliy Larin   *
 *   _junk@mail.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "wgl.h"

wGL::wGL(QWidget *parent, const char *name):QGLWidget(parent,name)
{
}

void wGL::initializeGL()
{
  this->maxDepth = 1;
  glClearColor(0,0,0,1);
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  baseColor.setRgb(255,255,255);
  coloredDepth = false;
}

void wGL::setMaxDepth(const unsigned char maxDepth)
{
  this->maxDepth = maxDepth;
  updateGL();
}

void wGL::setFill(const bool fill)
{
  if(fill)
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  else
   glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
  updateGL();
}

void wGL::setBaseColor(const QColor color)
{
  baseColor = color;
  updateGL();
}

void wGL::setColoredDepth(const bool colored)
{
  coloredDepth = colored;
  updateGL();
}

void paintTria()
{
  double h = sqrt(3.)/2;
  glBegin(GL_TRIANGLES);
  glVertex2f(0,h/3.*2);
  glVertex2f(-0.5,-h/3.);
  glVertex2f(0.5,-h/3.);
  glEnd();
}

const double TH = sqrt(3.)/2.;

void wGL::paintStarOfDavid(unsigned int iteration)
{
  if(iteration<maxDepth)
  {
    if(coloredDepth) 
      glColor3f(
        (double)baseColor.red()/255.*iteration/maxDepth,
        (double)baseColor.green()/255.*iteration/maxDepth,
        (double)baseColor.blue()/255.*iteration/maxDepth
      );

    glPushMatrix();
    glRotated(60, 0,0, -1);
    paintTria();
    glPopMatrix();

    //0 degree
    glPushMatrix();
    glTranslated(0, TH/3 + TH/9,0);
    glScaled(1/3.,1/3.,1/3.);
    paintStarOfDavid(iteration+1);
    glPopMatrix();

    //60 degree
    glPushMatrix();
    glRotated(60, 0,0, -1);  
    glTranslated(0, TH/3 + TH/9,0);
    glScaled(1/3.,1/3.,1/3.);
    paintStarOfDavid(iteration+1);
    glPopMatrix();

    //-60 degree
    glPushMatrix();
    glRotated(-60, 0,0, -1);  
    glTranslated(0, TH/3 + TH/9,0);
    glScaled(1/3.,1/3.,1/3.);
    paintStarOfDavid(iteration+1);
    glPopMatrix();

    //180 degree
    glPushMatrix();
    glRotated(180, 0,0, -1);  
    glTranslated(0, TH/3 + TH/9,0);
    glScaled(1/3.,1/3.,1/3.);
    paintStarOfDavid(iteration+1);
    glPopMatrix();

    //120 degree
    glPushMatrix();
    glRotated(120, 0,0, -1);  
    glTranslated(0, TH/3 + TH/9,0);
    glScaled(1/3.,1/3.,1/3.);
    paintStarOfDavid(iteration+1);
    glPopMatrix();

    //-120 degree
    glPushMatrix();
    glRotated(-120, 0,0, -1);  
    glTranslated(0, TH/3 + TH/9,0);
    glScaled(1/3.,1/3.,1/3.);
    paintStarOfDavid(iteration+1);
    glPopMatrix();
    
   /*glPushMatrix();
    glRotated(-180, 0,0, -1);  
    paintStarOfDavid(iteration+1);
    glPopMatrix();*/

 }
}

void wGL::paintGL()
{ 
  glClear(GL_COLOR_BUFFER_BIT);
 
  glColor3f((double)baseColor.red()/255.,(double)baseColor.green()/255.,(double)baseColor.blue()/255.);
  paintTria();
  paintStarOfDavid(1);
}
